#ifndef catalyst_conduit_blueprint_mangle_h
#define catalyst_conduit_blueprint_mangle_h

#define conduit_blueprint_about catalyst_conduit_blueprint_about
#define conduit_blueprint_mcarray_examples_xyz catalyst_conduit_blueprint_mcarray_examples_xyz
#define conduit_blueprint_mcarray_is_interleaved catalyst_conduit_blueprint_mcarray_is_interleaved
#define conduit_blueprint_mcarray_to_contiguous catalyst_conduit_blueprint_mcarray_to_contiguous
#define conduit_blueprint_mcarray_to_interleaved catalyst_conduit_blueprint_mcarray_to_interleaved
#define conduit_blueprint_mcarray_verify catalyst_conduit_blueprint_mcarray_verify
#define conduit_blueprint_mcarray_verify_sub_protocol catalyst_conduit_blueprint_mcarray_verify_sub_protocol
#define conduit_blueprint_mesh_examples_basic catalyst_conduit_blueprint_mesh_examples_basic
#define conduit_blueprint_mesh_examples_braid catalyst_conduit_blueprint_mesh_examples_braid
#define conduit_blueprint_mesh_examples_julia catalyst_conduit_blueprint_mesh_examples_julia
#define conduit_blueprint_mesh_examples_misc catalyst_conduit_blueprint_mesh_examples_misc
#define conduit_blueprint_mesh_examples_polytess catalyst_conduit_blueprint_mesh_examples_polytess
#define conduit_blueprint_mesh_examples_spiral catalyst_conduit_blueprint_mesh_examples_spiral
#define conduit_blueprint_mesh_generate_index catalyst_conduit_blueprint_mesh_generate_index
#define conduit_blueprint_mesh_verify catalyst_conduit_blueprint_mesh_verify
#define conduit_blueprint_mesh_verify_sub_protocol catalyst_conduit_blueprint_mesh_verify_sub_protocol
#define conduit_blueprint_verify catalyst_conduit_blueprint_verify

#endif
